/**
 * edg-gridftp-base-exists
 * 
 * A command line interface to the GridFTP libraries to allow a user
 * to determine if a file for directory exists on a remote server.
 * End users should invoke the perl wrapper rather than this.
 *
 * This command returns 0 if the given file or directory exists; it
 * 1 for any error.  The underlying error message from the GridFTP 
 * library should be printed on stderr. 
 *
 * Charles Loomis (2002-10-18)
 * CERN IT-DM/SMD (2008-02-25)
 * */

/* Include Files */
#include <stdlib.h>
#include <unistd.h>

#include "globus_ftp_client.h"
#include "edg-gridftp-base-common.h"

const char *prog;

/* Main function. */
int main(int argc, char **argv) {

  char *usage = "Usage: edg-gridftp-base-exists [-n] [-t <timeout in seconds>] <gsiftp URL>\n";

  char *url;
  globus_url_t url_struct;
  globus_bool_t dcauth; 
  globus_ftp_control_dcau_t dcau;

  int c;
  int status;
  int timeout;
  globus_result_t gresult;

  globus_ftp_client_handle_t ftp_handle;
  globus_ftp_client_handleattr_t ftp_handle_attr;
  globus_ftp_client_operationattr_t ftp_op_attr;

  monitor_t monitor;

  prog = argv[0];

 /* Check the arguments. Single ftp, gsiftp, or gridftp URL. 
  * Recognize a -n option for no data channel authentication.
  */
  dcauth = GLOBUS_TRUE;
  timeout = 120;

  opterr = 0;
  optind = 1;
  while((c = getopt(argc, argv, "nt:")) != -1) {
    switch(c) {
      case 'n':
          dcauth = GLOBUS_FALSE;
          break;
      case 't':
          timeout = atoi(optarg);
          if (timeout <= 0) {
            my_fprintf(stderr, "%s: invalid timeout %s\n", prog, optarg);
            return FAIL;
          }
          break;
      default:
          my_fprintf(stderr,usage);
          return FAIL;
          break;
    }
  }
  argc -= optind;
  argv += optind;

  if (argc != 1) {
    my_fprintf(stderr,usage);
    return FAIL;
  }
  url = argv[0];

  setup_timer_and_signals(timeout);

  /* Try parsing the argument to ensure that it is OK. */
  status = globus_url_parse(url, &url_struct);
  if (status != GLOBUS_SUCCESS) {
    my_fprintf(stderr, "%s: error parsing URL\n",prog);
    return FAIL;
  }

  /* Initialize the lock. */
  globus_mutex_init(&monitor.mutex, GLOBUS_NULL);
  globus_cond_init(&monitor.cond, GLOBUS_NULL);
  monitor.error = GLOBUS_FALSE;
  monitor.done = GLOBUS_FALSE;
  monitor.monitor = NULL;

  /* Activate the globus ftp client module. */
  status = globus_module_activate(GLOBUS_FTP_CLIENT_MODULE);
  if (status != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not activate ftp client module\n",
			prog);
    return FAIL;
  }


  /* Create an empty ftp client attribute. */
  gresult = globus_ftp_client_handleattr_init(&ftp_handle_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not create ftp handle attributes\n",
			prog);
    return FAIL;
  }


  /* Create an ftp client handle. */
  gresult = globus_ftp_client_handle_init(&ftp_handle,&ftp_handle_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
			"%s: could not create ftp handle\n",
			prog);
    return FAIL;
  }


  /* Create an ftp operations attribute. */
  gresult = globus_ftp_client_operationattr_init(&ftp_op_attr);
  if (gresult != GLOBUS_SUCCESS) {
    my_fprintf(stderr, 
  			"%s: could not create operations attribute\n",
  			prog);
    return FAIL;
  }
  
  /* If URL contains username and password, then set the authorization 
     to use this rather than the default GSI authorization. */
  if (url_struct.user || url_struct.password) {
    gresult = 
      globus_ftp_client_operationattr_set_authorization(&ftp_op_attr,
							GSS_C_NO_CREDENTIAL,
							url_struct.user,
							url_struct.password,
							"",
							"");
    if (gresult != GLOBUS_SUCCESS) {
      my_fprintf(stderr, 
			  "%s: could not set authorization info\n",
			  prog);
      return FAIL;
    }
  }

  /* If the user has requested that data channel authentication be
     turned off, do so. */
  if (! dcauth) {
    dcau.mode = GLOBUS_FTP_CONTROL_DCAU_NONE;
    gresult = 
      globus_ftp_client_operationattr_set_dcau(&ftp_op_attr,&dcau);
    if (gresult != GLOBUS_SUCCESS) {
      my_fprintf(stderr, 
			  "%s: could not turn off data channel auth\n",
			  prog);
      return FAIL;
    }
  }

  /* Now do the real work, does the file/directory exist? */
  gresult = globus_ftp_client_exists(&ftp_handle,
				     url,
				     &ftp_op_attr,
				     &callback,
				     (void *) &monitor);

  /* Wait for the result. */
  globus_mutex_lock(&monitor.mutex);
  while (!monitor.done) {
    globus_cond_wait(&monitor.cond, &monitor.mutex);
  }
  globus_mutex_unlock(&monitor.mutex);


  /* Clean up.  Silently ignore errors since we already have the result
     of the ftp operation itself. */
  gresult = globus_ftp_client_operationattr_destroy(&ftp_op_attr);
  gresult = globus_ftp_client_handle_destroy(&ftp_handle);
  gresult = globus_ftp_client_handleattr_destroy(&ftp_handle_attr);
  status = globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);

  return monitor.error ? FAIL : OK;

} /* main() */
